@@ -22,28 +22,11 @@ GetOptions(
    'directory|d=s'  => \( my $LANGUAGE_DIR ),
    'use-theme|U'    => \( my $USE_THEME ),
    'folding|F'      => \( my $FOLDING ),
+   'injections|J'   => \( my $INJECTIONS ),
 ) or exit 1;
 
 STDOUT->binmode( ':encoding(UTF-8)' );
 
-my $ts = Text::Treesitter->new(
-   lang_name => $LANGUAGE,
-   lang_dir  => $LANGUAGE_DIR,
-);
-
-my $query = $ts->load_query_file( "queries/highlights.scm" );
-
-my $query_folding;
-
-my $folding_path;
-if( $FOLDING and -f ( $folding_path = $ts->query_file_path( "queries/fold.scm" ) ) ) {
-   $query_folding = $ts->load_query_file( $folding_path );
-}
-
-my $str = String::Tagged->new( read_text $ARGV[0] // "/dev/stdin" );
-
-my $tree = $ts->parse_string( $str );
-
 my %FORMATS = (
    # Names stolen from tree-sitter's highlight theme
    attribute  => { fg => "vga:cyan", italic => 1 },
@@ -71,6 +54,7 @@ my %FORMATS = (
    'text.literal'  => { monospace => 1 },
    'text.quote'    => { italic => 1, bg => "xterm:236", },
    'text.strong'   => { bold => 1 },
+   'text.title'    => { fg => "vga:yellow", bold => 1, under => 1 },
    'text.uri'      => { fg => "vga:blue", under => 1 },
 
    # Extra names
@@ -79,7 +63,7 @@ my %FORMATS = (
    verbatim   => { fg => "xterm:251", monospace => 1 },
 );
 
-if( $USE_THEME and my $config = $ts->treesitter_config ) {
+if( $USE_THEME and my $config = Text::Treesitter->treesitter_config ) {
    my %theme = $config->{theme}->%*;
    foreach my $key ( sort keys %theme ) {
       my %format = ( ref $theme{$key} ) ? $theme{$key}->%* : ( color => $theme{$key} );
@@ -97,78 +81,125 @@ foreach ( values %FORMATS ) {
       $_->{bg} = Convert::Color->new( $_->{bg} )->as_xterm;
 }
 
-my $qc = Text::Treesitter::QueryCursor->new;
-
-# For ease of code management, line numbers are 0-indexed
+my $str = String::Tagged->new( read_text $ARGV[0] // "/dev/stdin" );
 
 my @fold_regions;
 my $fold_deepest = 0;
-if( $FOLDING and $query_folding ) {
-   $qc->exec( $query_folding, $tree->root_node );
-
-   my @regions_applied;
-
-   while( my $match = $qc->next_match ) {
-      my @captures = $match->captures;
-
-      next unless $query_folding->test_predicates_for_match( $tree, $match, \@captures );
-
-      my @nodes;
-
-      foreach my $capture ( @captures ) {
-         next unless $query_folding->capture_name_for_id( $capture->capture_id ) eq "fold";
-
-         push @nodes, $capture->node;
-      }
-
-      # Some fold patterns capture multiple toplevel nodes.
-      my $startline = ( $nodes[ 0]->start_point )[0];
-      my $endline   = ( $nodes[-1]->end_point )[0];
-
-      die "TODO: This fold region starts earlier than the previous one"
-         if @fold_regions and $startline < $fold_regions[-1][0];
-
-      next if $startline == $endline;
-
-      push @fold_regions, [ $startline, $endline ];
-
-      pop @regions_applied while @regions_applied and $regions_applied[-1][1] < $startline;
-      push @regions_applied, $fold_regions[-1];
-
-      $fold_deepest = scalar @regions_applied if @regions_applied > $fold_deepest;
-   }
-}
-
-$qc->exec( $query, $tree->root_node );
 
 my %UNRECOGNISED_CAPTURES;
 
-while( my $match = $qc->next_match ) {
-   my @captures = $match->captures;
+my %TS_FOR_LANGUAGE;
 
-   next unless $query->test_predicates_for_match( $tree, $match, \@captures );
+sub apply_language_highlights( $language, %opts )
+{
+   my $ts = $TS_FOR_LANGUAGE{ $language } //= eval { Text::Treesitter->new(
+      lang_name => $language,
+      lang_dir  => $LANGUAGE_DIR,
+   ) };
+   defined $ts or return;
 
-   CAPTURE: foreach my $capture ( @captures ) {
-      my $node = $capture->node;
-      my $capturename = $query->capture_name_for_id( $capture->capture_id );
+   my $query_highlight = $ts->load_query_file( "highlights.scm" );
 
-      my $start = $tree->byte_to_char( $node->start_byte );
-      my $len   = $tree->byte_to_char( $node->end_byte ) - $start;
+   my $query_folding;
+   if( $FOLDING and -f ( my $folding_path = $ts->query_file_path( "fold.scm" ) ) ) {
+      $query_folding = $ts->load_query_file( $folding_path );
+   }
 
-      my @nameparts = split m/\./, $capturename;
-      while( @nameparts ) {
-         if( my $format = $FORMATS{ join ".", @nameparts } ) {
-            $str->apply_tag( $start, $len, $_, $format->{$_} ) for keys %$format;
-            next CAPTURE;
+   my $query_injections;
+   if( $INJECTIONS and -f ( my $injections_path = $ts->query_file_path( "injections.scm" ) ) ) {
+      $query_injections = $ts->load_query_file( $injections_path );
+   }
+
+   my $tree;
+   if( defined $opts{start_byte} ) {
+      $tree = $ts->parse_string_range( $str, %opts{qw( start_byte end_byte )} );
+   }
+   else {
+      $tree = $ts->parse_string( $str );
+   }
+   my $root = $tree->root_node;
+
+   my $qc = Text::Treesitter::QueryCursor->new;
+
+   # For ease of code management, line numbers are 0-indexed
+   if( $FOLDING and $query_folding ) {
+      $qc->exec( $query_folding, $root );
+
+      my @regions_applied;
+
+      while( my $captures = $qc->next_match_captures( multi => 1 ) ) {
+         my @nodes = $captures->{fold}->@*;
+
+         # Some fold patterns capture multiple toplevel nodes.
+         my $startline = ( $nodes[ 0]->start_point )[0];
+         my $endline   = ( $nodes[-1]->end_point )[0];
+
+         die "TODO: This fold region starts earlier than the previous one"
+            if @fold_regions and $startline < $fold_regions[-1][0];
+
+         next if $startline == $endline;
+
+         push @fold_regions, [ $startline, $endline ];
+
+         pop @regions_applied while @regions_applied and $regions_applied[-1][1] < $startline;
+         push @regions_applied, $fold_regions[-1];
+
+         $fold_deepest = scalar @regions_applied if @regions_applied > $fold_deepest;
+      }
+   }
+
+   if( $INJECTIONS and $query_injections ) {
+      $qc->exec( $query_injections, $root );
+
+      while( my $captures = $qc->next_match_captures ) {
+         my $sublanguage;
+         my $content;
+         if( defined $captures->{language} ) {
+            $sublanguage = $captures->{language}->text;
+            $content     = $captures->{content};
+         }
+         elsif( keys $captures->%* > 1 ) {
+            warn "This injection capture yielded more than one name key\n";
+            next;
+         }
+         else {
+            $sublanguage = ( keys $captures->%* )[0];
+            $content     = $captures->{$sublanguage};
          }
 
-         pop @nameparts;
+         apply_language_highlights( $sublanguage,
+            start_byte => $content->start_byte,
+            end_byte   => $content->end_byte,
+         );
       }
+   }
 
-      $UNRECOGNISED_CAPTURES{ $capturename }++;
+   $qc->exec( $query_highlight, $root );
+
+   while( my $captures = $qc->next_match_captures ) {
+      CAPTURE: foreach my $capturename ( sort keys $captures->%* ) {
+         my $node = $captures->{$capturename};
+
+         my $start = $tree->byte_to_char( $node->start_byte );
+         my $len   = $tree->byte_to_char( $node->end_byte ) - $start;
+
+         my @nameparts = split m/\./, $capturename;
+         while( @nameparts ) {
+            if( my $format = $FORMATS{ join ".", @nameparts } ) {
+               $str->apply_tag( $start, $len, $_, $format->{$_} ) for keys %$format;
+               next CAPTURE;
+            }
+
+            pop @nameparts;
+         }
+
+         $UNRECOGNISED_CAPTURES{ $capturename }++;
+      }
    }
 }
 
+apply_language_highlights( $LANGUAGE );
+
 foreach my ( $lnum, $line ) ( indexed $str->split( qr/\n/ ) ) {
    if( $FOLDING ) {
       my @regions = grep { $_->[0] <= $lnum and $lnum <= $_->[1] } @fold_regions;
